@@ -1,6 +1,8 @@
 package status
 
 import (
+	"context"
+	"errors"
 	"fmt"
 	"sync"
 	"time"
@@ -11,20 +13,19 @@ import (
 	operatorhelpers "github.com/openshift/library-go/pkg/operator/v1helpers"
 	olm "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
 	mktconfig "github.com/operator-framework/operator-marketplace/pkg/apis/config/v1"
-	"github.com/operator-framework/operator-marketplace/pkg/apis/operators/v1"
-	"github.com/operator-framework/operator-marketplace/pkg/apis/operators/v2"
+	v1 "github.com/operator-framework/operator-marketplace/pkg/apis/operators/v1"
+	v2 "github.com/operator-framework/operator-marketplace/pkg/apis/operators/v2"
+	"github.com/operator-framework/operator-marketplace/pkg/defaults"
 	"github.com/operator-framework/operator-marketplace/pkg/operatorhub"
 	log "github.com/sirupsen/logrus"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/client-go/rest"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 )
 
 const (
-	// clusterOperatorName is the name of the ClusterOperator
-	clusterOperatorName = "marketplace"
-
 	// minSyncsBeforeReporting is the minimum number of syncs we wish to see
 	// before reporting that the operator is available
 	minSyncsBeforeReporting = 3
@@ -47,18 +48,29 @@ const (
 	// coStatusReportInterval is the interval at which the ClusterOperator status is updated
 	coStatusReportInterval = 20 * time.Second
 
-	// Marketplace is always upgradeable and should include this message in the Upgradeable
-	// ClusterOperatorStatus condition.
+	// Since Openshift 4.4, Marketplace is only upgradeable if there are no custom (non-default)
+	// OperatorSources or CatalogSourceConifgs, and includes either a upgradeable message
+	// or a deprecatedAPIMessage in the Upgradeable ClusterOperatorStatus condition, conditioned
+	// on the presence of custom resources
+
 	upgradeableMessage = "Marketplace is upgradeable"
+
+	deprecatedAPIMessage = "The cluster has custom OperatorSource/CatalogSourceConfig, which are deprecated in future versions. Please visit this link for further deatils: https://docs.openshift.com/container-platform/4.4/release_notes/ocp-4-4-release-notes.html#ocp-4-4-marketplace-apis-deprecated"
 )
 
-// status will be a singleton
-var instance *status
-var once sync.Once
+type SyncSender interface {
+	SendSyncMessage(err error)
+}
 
-type status struct {
+type Reporter interface {
+	StartReporting() <-chan struct{}
+	ReportMigration() error
+	SyncSender
+}
+
+type reporter struct {
 	configClient    *configclient.ConfigV1Client
-	coAPINotPresent bool
+	rawClient       client.Client
 	namespace       string
 	clusterOperator *configv1.ClusterOperator
 	version         string
@@ -68,112 +80,35 @@ type status struct {
 	// stopCh is used to signal that threads should stop reporting ClusterOperator status
 	stopCh <-chan struct{}
 	// monitorDoneCh is used to signal that threads are done reporting ClusterOperator status
-	monitorDoneCh chan struct{}
-}
-
-// SendSyncMessage is used to send messages to the syncCh. If the channel is
-// busy, the sync will be dropped to prevent the controller from stalling.
-func SendSyncMessage(err error) {
-	// If the coAPI is not available do not attempt to send messages to the
-	// sync channel
-	if instance == nil || instance.coAPINotPresent {
-		return
-	}
-	// A missing sync status is better than stalling the controller
-	select {
-	case instance.syncCh <- err:
-		log.Debugf("[status] Sent message to the sync channel")
-		break
-	default:
-		log.Debugf("[status] Sync channel is busy, not reporting sync")
-	}
-}
-
-// StartReporting ensures that the cluster supports reporting ClusterOperator status
-// and returns a channel that reports if it is actively reporting.
-func StartReporting(cfg *rest.Config, mgr manager.Manager, namespace string, version string, stopCh <-chan struct{}) <-chan struct{} {
-	// ensure instance is only created once.
-	once.Do(func() {
-		instance = new(cfg, mgr, namespace, version, stopCh)
-		// exit if ClusterOperator api is not present
-		if instance.coAPINotPresent {
-			return
-		}
-
-		// start consuming messages on the sync channel
-		go instance.syncChannelReceiver()
-
-		// start reporting ClusterOperator status
-		go instance.monitorClusterStatus()
-	})
-
-	return instance.monitorDoneCh
-}
-
-// New returns an initialized status
-func new(cfg *rest.Config, mgr manager.Manager, namespace string, version string, stopCh <-chan struct{}) *status {
-	monitorDoneCh := make(chan struct{})
-	coAPINotPresent := !mktconfig.IsAPIAvailable()
-	if coAPINotPresent {
-		log.Warningf("[status] ClusterOperator API not present")
-		// If the co is not present, close the monitorDoneCh channel to prevent
-		// receivers from stalling
-		close(monitorDoneCh)
-	}
-
-	// Client for handling reporting of operator status
-	configClient, err := configclient.NewForConfig(cfg)
-	if err != nil {
-		log.Fatal("[status] " + err.Error())
-	}
-
-	syncRatio, err := NewSyncRatio(successRatio, syncsBeforeTruncate, syncTruncateValue)
-	if err != nil {
-		log.Fatal("[status] " + err.Error())
-	}
-
-	// If version is an empty string, warn that the operator is not a part of the OpenShift release payload.
-	if version == "" {
-		version = "OpenShift Independent Version"
-	}
-
-	return &status{
-		configClient:    configClient,
-		coAPINotPresent: coAPINotPresent,
-		namespace:       namespace,
-		version:         version,
-		syncRatio:       syncRatio,
-		// Add a buffer to prevent dropping syncs
-		syncCh:        make(chan error, 25),
-		stopCh:        stopCh,
-		monitorDoneCh: monitorDoneCh,
-	}
+	monitorDoneCh       chan struct{}
+	clusterOperatorName string
+	once                sync.Once
 }
 
 // ensureClusterOperator ensures that a ClusterOperator CR is present on the
 // cluster
-func (s *status) ensureClusterOperator() error {
+func (r *reporter) ensureClusterOperator() error {
 	var err error
-	s.clusterOperator, err = s.configClient.ClusterOperators().Get(clusterOperatorName, metav1.GetOptions{})
+	r.clusterOperator, err = r.configClient.ClusterOperators().Get(r.clusterOperatorName, metav1.GetOptions{})
 
 	if err == nil {
 		log.Debug("[status] Found existing ClusterOperator")
 		return nil
 	}
 
-	if err != nil && !apierrors.IsNotFound(err) {
+	if !apierrors.IsNotFound(err) {
 		return fmt.Errorf("Error %v getting ClusterOperator", err)
 	}
 
 	clusterOperator := &configv1.ClusterOperator{
 		ObjectMeta: metav1.ObjectMeta{
-			Name:      clusterOperatorName,
-			Namespace: s.namespace,
+			Name:      r.clusterOperatorName,
+			Namespace: r.namespace,
 		},
 	}
-	s.setRelatedObjects()
+	r.setRelatedObjects()
 
-	s.clusterOperator, err = s.configClient.ClusterOperators().Create(clusterOperator)
+	r.clusterOperator, err = r.configClient.ClusterOperators().Create(clusterOperator)
 	if err != nil {
 		return fmt.Errorf("Error %v creating ClusterOperator", err)
 	}
@@ -183,20 +118,17 @@ func (s *status) ensureClusterOperator() error {
 
 // setStatus handles setting all the required fields for the given
 // ClusterStatusConditionType
-func (s *status) setStatus(statusConditions []configv1.ClusterOperatorStatusCondition) error {
-	if s.coAPINotPresent {
-		return fmt.Errorf("ClusterOperator API not present")
-	}
-	err := s.ensureClusterOperator()
+func (r *reporter) setStatus(statusConditions []configv1.ClusterOperatorStatusCondition) error {
+	err := r.ensureClusterOperator()
 	if err != nil {
 		return err
 	}
-	previousStatus := s.clusterOperator.Status.DeepCopy()
+	previousStatus := r.clusterOperator.Status.DeepCopy()
 	for _, statusCondition := range statusConditions {
-		s.setStatusCondition(statusCondition)
+		r.setStatusCondition(statusCondition)
 	}
 
-	err = s.updateStatus(previousStatus)
+	err = r.updateStatus(previousStatus)
 	if err != nil {
 		return err
 	}
@@ -206,35 +138,35 @@ func (s *status) setStatus(statusConditions []configv1.ClusterOperatorStatusCond
 // setOperandVersion sets the operator version in the ClusterOperator Status
 // Per instructions from the CVO team, setOperandVersion should only be called
 // when the operator becomes available
-func (s *status) setOperandVersion() {
+func (r *reporter) setOperandVersion() {
 	// Report the operator version
 	operatorVersion := configv1.OperandVersion{
 		Name:    "operator",
-		Version: s.version,
+		Version: r.version,
 	}
-	operatorhelpers.SetOperandVersion(&s.clusterOperator.Status.Versions, operatorVersion)
+	operatorhelpers.SetOperandVersion(&r.clusterOperator.Status.Versions, operatorVersion)
 }
 
 // setStatusCondition sets the operator StatusCondition in the ClusterOperator Status
-func (s *status) setStatusCondition(statusCondition configv1.ClusterOperatorStatusCondition) {
+func (r *reporter) setStatusCondition(statusCondition configv1.ClusterOperatorStatusCondition) {
 	// Only update the version when the operator becomes available
 	if statusCondition.Type == configv1.OperatorAvailable && statusCondition.Status == configv1.ConditionTrue {
-		s.setOperandVersion()
+		r.setOperandVersion()
 	}
-	cohelpers.SetStatusCondition(&s.clusterOperator.Status.Conditions, statusCondition)
+	cohelpers.SetStatusCondition(&r.clusterOperator.Status.Conditions, statusCondition)
 }
 
 // updateStatus makes the API call to update the ClusterOperator if the status has changed.
-func (s *status) updateStatus(previousStatus *configv1.ClusterOperatorStatus) error {
+func (r *reporter) updateStatus(previousStatus *configv1.ClusterOperatorStatus) error {
 	var err error
-	if compareClusterOperatorStatusConditionArrays(previousStatus.Conditions, s.clusterOperator.Status.Conditions) {
+	if compareClusterOperatorStatusConditionArrays(previousStatus.Conditions, r.clusterOperator.Status.Conditions) {
 		log.Debugf("[status] Previous and current ClusterOperator Status are the same, the ClusterOperator Status will not be updated.")
 	} else {
 		log.Debugf("[status] Previous and current ClusterOperator Status are different, attempting to update the ClusterOperator Status.")
 
 		// Check if the ClusterOperator version has changed and log the attempt to upgrade if it has
 		previousVersion := operatorhelpers.FindOperandVersion(previousStatus.Versions, "operator")
-		currentVersion := operatorhelpers.FindOperandVersion(s.clusterOperator.Status.Versions, "operator")
+		currentVersion := operatorhelpers.FindOperandVersion(r.clusterOperator.Status.Versions, "operator")
 		if currentVersion != nil {
 			if previousVersion == nil {
 				log.Infof("[status] Attempting to set ClusterOperator to version %s", currentVersion.Version)
@@ -245,14 +177,14 @@ func (s *status) updateStatus(previousStatus *configv1.ClusterOperatorStatus) er
 
 		// Log Conditions
 		log.Infof("[status] Attempting to set the ClusterOperator status conditions to:")
-		for _, statusCondition := range s.clusterOperator.Status.Conditions {
+		for _, statusCondition := range r.clusterOperator.Status.Conditions {
 			log.Infof("[status] ConditionType: %v ConditionStatus: %v ConditionMessage: %v", statusCondition.Type, statusCondition.Status, statusCondition.Message)
 		}
 
 		// Always update RelatedObjects to account for the upgrade case.
-		s.setRelatedObjects()
+		r.setRelatedObjects()
 
-		_, err := s.configClient.ClusterOperators().UpdateStatus(s.clusterOperator)
+		_, err := r.configClient.ClusterOperators().UpdateStatus(r.clusterOperator)
 		if err != nil {
 			return fmt.Errorf("Error %v updating ClusterOperator", err)
 		}
@@ -263,49 +195,49 @@ func (s *status) updateStatus(previousStatus *configv1.ClusterOperatorStatus) er
 
 // setRelatedObjects populates RelatedObjects in the ClusterOperator.Status.
 // RelatedObjects are consumed by https://github.com/openshift/must-gather
-func (s *status) setRelatedObjects() {
+func (r *reporter) setRelatedObjects() {
 	objectReferences := []configv1.ObjectReference{
 		// Add the operator's namespace which will result in core resources
 		// being gathered
 		{
 			Resource: "namespaces",
-			Name:     s.namespace,
+			Name:     r.namespace,
 		},
 		// Add the non-core resources we care about
 		{
 			Group:     v1.SchemeGroupVersion.Group,
 			Resource:  v1.OperatorSourceKind,
-			Namespace: s.namespace,
+			Namespace: r.namespace,
 		},
 		{
 			Group:     v2.SchemeGroupVersion.Group,
 			Resource:  v2.CatalogSourceConfigKind,
-			Namespace: s.namespace,
+			Namespace: r.namespace,
 		},
 		{
 			Group:     olm.GroupName,
 			Resource:  olm.CatalogSourceKind,
-			Namespace: s.namespace,
+			Namespace: r.namespace,
 		},
 	}
-	s.clusterOperator.Status.RelatedObjects = objectReferences
+	r.clusterOperator.Status.RelatedObjects = objectReferences
 }
 
 // syncChannelReceiver will listen on the sync channel and update the status
 // syncsRatio filed until the stopCh is closed.
-func (s *status) syncChannelReceiver() {
+func (r *reporter) syncChannelReceiver() {
 	log.Info("[status] Starting sync consumer")
 	for {
 		select {
-		case <-s.stopCh:
+		case <-r.stopCh:
 			return
-		case err := <-s.syncCh:
+		case err := <-r.syncCh:
 			if err == nil {
-				s.syncRatio.ReportSyncEvent()
+				r.syncRatio.ReportSyncEvent()
 			} else {
-				s.syncRatio.ReportFailedSync()
+				r.syncRatio.ReportFailedSync()
 			}
-			failedSyncs, syncs := s.syncRatio.GetSyncs()
+			failedSyncs, syncs := r.syncRatio.GetSyncs()
 			log.Debugf("[status] Faild Syncs / Total Syncs : %d/%d", failedSyncs, syncs)
 		}
 	}
@@ -313,23 +245,31 @@ func (s *status) syncChannelReceiver() {
 
 // monitorClusterStatus updates the ClusterOperator's status based on
 // the number of successful syncs / total syncs
-func (s *status) monitorClusterStatus() {
+func (r *reporter) monitorClusterStatus() {
 	// Signal to the main channel that we have stopped reporting status.
 	defer func() {
-		close(s.monitorDoneCh)
+		close(r.monitorDoneCh)
 	}()
 	for {
+		operatorUpgradeable, err := CheckOperatorUpgradeablity(r.rawClient)
+		if err != nil {
+			log.Errorf("Could not determine operator upgradeablity. Error: %s", err.Error())
+		}
 		select {
-		case <-s.stopCh:
+		case <-r.stopCh:
 			// If the stopCh is closed, set all ClusterOperatorStatus conditions to false.
 			reason := "OperatorExited"
 			msg := "The operator has exited"
 			conditionListBuilder := clusterStatusListBuilder()
 			conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionFalse, msg, reason)
 			conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionFalse, msg, reason)
-			conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
+			if operatorUpgradeable {
+				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
+			} else {
+				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionFalse, deprecatedAPIMessage, "DprecatedAPIsInUse")
+			}
 			statusConditions := conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, msg, reason)
-			statusErr := s.setStatus(statusConditions)
+			statusErr := r.setStatus(statusConditions)
 			if statusErr != nil {
 				log.Error("[status] " + statusErr.Error())
 			}
@@ -347,28 +287,36 @@ func (s *status) monitorClusterStatus() {
 
 			// Create the ClusterOperator in the progressing state if it does not exist
 			// or if it is the first report.
-			if s.clusterOperator == nil {
+			if r.clusterOperator == nil {
 				reason := "OperatorStarting"
 				conditionListBuilder := clusterStatusListBuilder()
-				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionTrue, fmt.Sprintf("Progressing towards release version: %s", s.version), reason)
-				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
+				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionTrue, fmt.Sprintf("Progressing towards release version: %s", r.version), reason)
+				if operatorUpgradeable {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
+				} else {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionFalse, deprecatedAPIMessage, "DprecatedAPIsInUse")
+				}
 				msg := fmt.Sprintf("Determining status")
 				conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionFalse, msg, reason)
 				statusConditions := conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, msg, reason)
-				statusErr = s.setStatus(statusConditions)
+				statusErr = r.setStatus(statusConditions)
 				break
 			}
 
-			_, syncEvents := s.syncRatio.GetSyncs()
+			_, syncEvents := r.syncRatio.GetSyncs()
 
 			// no default operator sources are present, so assume we are in a good state
 			if operatorhub.GetSingleton().Disabled() && syncEvents == 0 {
 				reason := "NoDefaultOpSrcEnabled"
 				conditionListBuilder := clusterStatusListBuilder()
-				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionFalse, fmt.Sprintf("Successfully progressed to release version: %s", s.version), reason)
-				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
-				statusConditions := conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionTrue, fmt.Sprintf("Available release version: %s", s.version), reason)
-				statusErr = s.setStatus(statusConditions)
+				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionFalse, fmt.Sprintf("Successfully progressed to release version: %s", r.version), reason)
+				if operatorUpgradeable {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
+				} else {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionFalse, deprecatedAPIMessage, "DprecatedAPIsInUse")
+				}
+				statusConditions := conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionTrue, fmt.Sprintf("Available release version: %s", r.version), reason)
+				statusErr = r.setStatus(statusConditions)
 				break
 			}
 
@@ -379,43 +327,175 @@ func (s *status) monitorClusterStatus() {
 			}
 
 			// Report that marketplace is available after meeting minimal syncs.
-			if cohelpers.IsStatusConditionFalse(s.clusterOperator.Status.Conditions, configv1.OperatorAvailable) {
+			if cohelpers.IsStatusConditionFalse(r.clusterOperator.Status.Conditions, configv1.OperatorAvailable) {
 				reason := "OperatorAvailable"
 				conditionListBuilder := clusterStatusListBuilder()
-				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionFalse, fmt.Sprintf("Successfully progressed to release version: %s", s.version), reason)
-				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
-				statusConditions := conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionTrue, fmt.Sprintf("Available release version: %s", s.version), reason)
-				statusErr = s.setStatus(statusConditions)
+				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionFalse, fmt.Sprintf("Successfully progressed to release version: %s", r.version), reason)
+				if operatorUpgradeable {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
+				} else {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionFalse, deprecatedAPIMessage, "DprecatedAPIsInUse")
+				}
+				statusConditions := conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionTrue, fmt.Sprintf("Available release version: %s", r.version), reason)
+				statusErr = r.setStatus(statusConditions)
 				break
 			}
 
 			// Update the status with the appropriate state.
-			isSucceeding, ratio := s.syncRatio.IsSucceeding()
+			isSucceeding, ratio := r.syncRatio.IsSucceeding()
 			if ratio != nil {
 				var statusConditions []configv1.ClusterOperatorStatusCondition
 				conditionListBuilder := clusterStatusListBuilder()
-				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, "OperatorAvailable")
+				if operatorUpgradeable {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, "OperatorAvailable")
+				} else {
+					conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionFalse, deprecatedAPIMessage, "DprecatedAPIsInUse")
+				}
 				if isSucceeding {
 					statusConditions = conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, fmt.Sprintf("Current CR sync ratio (%g) meets the expected success ratio (%g)", *ratio, successRatio), "OperandTransitionsSucceeding")
 				} else {
 					statusConditions = conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionTrue, fmt.Sprintf("Current CR sync ratio (%g) does not meet the expected success ratio (%g)", *ratio, successRatio), "OperandTransitionsFailing")
 				}
-				statusErr = s.setStatus(statusConditions)
+				statusErr = r.setStatus(statusConditions)
 				break
 			}
 		}
 	}
 }
 
+func NewReporter(cfg *rest.Config, mgr manager.Manager, namespace string, name string, version string, stopCh <-chan struct{}) (Reporter, error) {
+	if !mktconfig.IsAPIAvailable() {
+		return nil, errors.New("[status] ClusterOperator API not present")
+	}
+
+	// Client for handling reporting of operator status
+	configClient, err := configclient.NewForConfig(cfg)
+	if err != nil {
+		return nil, fmt.Errorf("failed to create config v1 client: %s", err.Error())
+	}
+	// Client for listing OperatorSources and CatalogSourceConfigs
+	rawClient, err := client.New(cfg, client.Options{})
+	if err != nil {
+		return nil, fmt.Errorf("failed to create raw client: %s", err.Error())
+	}
+	syncRatio, err := NewSyncRatio(successRatio, syncsBeforeTruncate, syncTruncateValue)
+	if err != nil {
+		return nil, fmt.Errorf("failed to create status sync ratio: %s", err.Error())
+	}
+
+	// If version is an empty string, warn that the operator is not a part of the OpenShift release payload.
+	if version == "" {
+		version = "OpenShift Independent Version"
+	}
+
+	return &reporter{
+		configClient: configClient,
+		rawClient:    rawClient,
+		namespace:    namespace,
+		version:      version,
+		syncRatio:    syncRatio,
+		// Add a buffer to prevent dropping syncs
+		syncCh:              make(chan error, 25),
+		stopCh:              stopCh,
+		monitorDoneCh:       make(chan struct{}),
+		clusterOperatorName: name,
+	}, nil
+}
+
 // ReportMigration sets the clusterOperator status to signal that migration logic is in progress,
 // while upgrading from openshift 4.1.z to openshift 4.2.z
-func ReportMigration(cfg *rest.Config, mgr manager.Manager, namespace string, version string, stopCh <-chan struct{}) error {
-	instance = new(cfg, mgr, namespace, version, stopCh)
+func (r *reporter) ReportMigration() error {
 	conditionListBuilder := clusterStatusListBuilder()
-	conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionTrue, fmt.Sprintf("Performing migration logic to progress towards release version: %s", instance.version), "Upgrading")
+	conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionTrue, fmt.Sprintf("Performing migration logic to progress towards release version: %s", r.version), "Upgrading")
 	msg := fmt.Sprintf("Determining status")
 	conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionFalse, msg, "Upgrading")
 	conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionFalse, msg, "Upgrading")
 	statusConditions := conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, msg, "Upgrading")
-	return instance.setStatus(statusConditions)
+	return r.setStatus(statusConditions)
+}
+
+// StartReporting ensures that the cluster supports reporting ClusterOperator status
+// and returns a channel that reports if it is actively reporting.
+func (r *reporter) StartReporting() <-chan struct{} {
+	// ensure each goroutine is only started once.
+	r.once.Do(func() {
+		// start consuming messages on the sync channel
+		go r.syncChannelReceiver()
+
+		// start reporting ClusterOperator status
+		go r.monitorClusterStatus()
+	})
+	return r.monitorDoneCh
+}
+
+// SendSyncMessage is used to send messages to the syncCh. If the channel is
+// busy, the sync will be dropped to prevent the controller from stalling.
+func (r *reporter) SendSyncMessage(err error) {
+	// A missing sync status is better than stalling the controller
+	select {
+	case r.syncCh <- err:
+		log.Debugf("[status] Sent message to the sync channel")
+		break
+	default:
+		log.Debugf("[status] Sync channel is busy, not reporting sync")
+	}
+}
+
+type NoOpReporter struct{}
+
+func (NoOpReporter) SendSyncMessage(err error) {
+}
+
+func (NoOpReporter) StartReporting() <-chan struct{} {
+	ch := make(chan struct{})
+	close(ch)
+	return ch
+}
+
+func (NoOpReporter) ReportMigration() error {
+	return nil
+}
+
+// CheckOperatorUpgradeablity checks for the presence of CatalogSourceConfigs,
+// and non default OperatorSources in the cluster. Since both the APIs are
+// scheduled to be deprecated/are already deprecated, checkOperatorUpgradeablity
+// returns false if it detects the presence of any CSC or non default Opsrc in
+// the cluster to indicate that the cluster is not upgradable to future versions.
+func CheckOperatorUpgradeablity(kubeClient client.Client) (bool, error) {
+	clusterHasCSC, err := isCSCPresent(kubeClient)
+	if err != nil {
+		return false, err
+	}
+	clusterHasOpsrc, err := isNonDefaultOpsrcPresent(kubeClient)
+	if err != nil {
+		return false, err
+	}
+	if clusterHasCSC || clusterHasOpsrc {
+		return false, nil
+	}
+	return true, nil
+}
+
+func isCSCPresent(kubeClient client.Client) (bool, error) {
+	cscs := &v2.CatalogSourceConfigList{}
+	if err := kubeClient.List(context.TODO(), &client.ListOptions{}, cscs); err != nil {
+		return false, err
+	}
+	if len(cscs.Items) > 0 {
+		return true, nil
+	}
+	return false, nil
+}
+
+func isNonDefaultOpsrcPresent(kubeClient client.Client) (bool, error) {
+	opsrcs := &v1.OperatorSourceList{}
+	if err := kubeClient.List(context.TODO(), &client.ListOptions{}, opsrcs); err != nil {
+		return false, err
+	}
+	for _, opsrc := range opsrcs.Items {
+		if !defaults.IsDefaultSource(opsrc.Name) {
+			return true, nil
+		}
+	}
+	return false, nil
 }
